#!/usr/bin python

import os
import sys
import re
import time
import traceback
import wx

#from pytronix.lib.log     import PytronixLog
from pytronix.lib.general import Tags
from pytronix.lib.general import Singleton
from pytronix.lib.gui  import PytronixGui as Gui

#log      = PytronixLog.PytronixLog    (__file__)

main_data_tags = [
    'tMainTitle',
    'tMainSize',
    'tMainPosition',
    'tMainButtonSizeX',
    'tMainButtonSizeY',
    'tMainViewSizeZ',
    'tMainMaxOutputPanelLines',
    'tMainTimerPollingMsec',
    ]

view_data_tags = [
    'tViewDualListsOn',
    'tViewDualListsProportion',
    ]

list_data_tags = [
    'tListHeaders',
    'tListProportions',
    ]


class PytronixGuiBoard_MainAbstract( object ):
    def __init__(self, base_board):
        self.base_board = base_board
        Tags.add_tags_as_attributes( self, main_data_tags)
        self.default_settings = {
            self.tMainTitle                : 'GuiBoard',
            self.tMainSize                 : (830, 590),
            self.tMainPosition             : ( 20,  20),
            self.tMainButtonSizeX          :          5,
            self.tMainButtonSizeY          :          4,
            self.tMainViewSizeZ            :          1,
            self.tMainMaxOutputPanelLines  :        200,
            self.tMainTimerPollingMsec     :          0,
            }
        self.settings = self.default_settings.copy()
        self.bindings = [
            [wx.EVT_CLOSE, self.binding_evt_close],
            ]
        self.pre_initialize_call = None
        self.post_initialize_call = None
                
        
    def update_settings(self, new_settings):
        self.settings.update(new_settings)

        
    def pre_initialize(self):
        if self.pre_initialize_call:
            self.pre_initialize_call()


    def post_initialize(self):
        if self.post_initialize_call:
            self.post_initialize_call()


    def build(self):
        pass


    def overload_close(self):
        pass


    def binding_evt_close(self, event):
        self.overload_close()
        self.base_board.gui.Destroy()



class PytronixGuiBoard_ViewAbstract( object ):
    def __init__(self, base_board):
        self.base_board = base_board
        Tags.add_tags_as_attributes( self, view_data_tags)
        self.main_obj = None
        self.gui      = None
        self.settings = {}
        self.list_1 = None
        self.list_2 = None
        self.header = '?'

        self.default_settings = {
            self.tViewDualListsOn         : False,
            self.tViewDualListsProportion : [50,50]
            }
        self.settings = self.default_settings.copy()


    def update_settings(self, new_settings):
        self.settings.update(new_settings)

    def build(self):
        self.gui = self.base_board.gui
        if self.settings[self.tViewDualListsOn]:
            self.list_1.build(do_not_add=1)
            self.list_2.build(0,self.list_1.list_obj.wxListCtrl,
                self.settings[self.tViewDualListsProportion])
            self.list_1.other_side_list = self.list_2
            self.list_2.other_side_list = self.list_1
        else:
            self.list_1.build()
            

class PytronixGuiBoard_ListAbstract( object ):
    def __init__(self, base_board):
        self.base_board = base_board
        Tags.add_tags_as_attributes( self, list_data_tags)
        self.list_obj = None
        self.default_settings = {
            self.tListHeaders         : '?',
            self.tListProportions     : [ 30 ],
            }
        self.settings = self.default_settings.copy()
        self.other_side_list = None


    def update_settings(self, new_settings):
        self.settings.update(new_settings)
        

        
    def build(self, do_not_add=0, added_wxListCtrl= 0, proportion=[50,50]):
        self.gui = self.base_board.gui
        if not self.list_obj:
            self.list_obj = Gui.PytronixGuiWin_List(self.gui, 'alist')
            self.list_obj.headers          = self.settings[self.tListHeaders]
            self.list_obj.widths           = self.settings[self.tListProportions]
            self.list_obj.selectFunction   = self.select_item
            self.list_obj.visibleMode      = 1
            self.list_obj.visibleOn        = 1
            if do_not_add:
                self.list_obj.build(do_not_add = 1)
                self.list_obj.proportion       = proportion[0]
            else:
                self.list_obj.proportion       = proportion[1]
                if added_wxListCtrl:
                    self.list_obj.build(0, added_wxListCtrl, proportion)
                else:
                    self.list_obj.build(0)
                    
            self.gui_list = []

            
    def select_item(self, event):
        self.selected_indices =  self.list_obj.getSelectedIndices()
        if self.other_side_list:
            self.other_side_list.clear_selected_indices()
        #log.info('S %s'%(self.selected_indices))


    def clear_selected_indices(self):
        self.selected_indices =  []
        self.list_obj.clear_selected()

        
        
    def update_list(self, gui_list ):
        self.gui_list = gui_list
        self.list_obj.fillText(self.gui_list)
        self.clear_selected_indices()




class PytronixGuiBoard_ListDirectory( PytronixGuiBoard_ListAbstract ):
    def __init__(self, base_board):
        PytronixGuiBoard_ListAbstract.__init__(self,base_board)
        width_total    = base_board.mdata(base_board.tMainSize)[0]/2 - 20
        width_size     = 50
        width_modified = 100
        width_cut      = 5
        width_dir      = width_total - width_size - width_modified - width_cut
        self.update_settings({
            self.tListHeaders     : [ 'Dir:','Size', 'Modified'],
            self.tListProportions : [ width_dir, width_size, width_modified],
            #self.tListProportions : [ 300, 100, 100],
            })
        self.format_active = 0
        self.format_size   = 1
        self.format_date   = 1
        self.directory     = ''
        self.active_files  = []
        self.pass_filters   = [ re.compile('^[^~]+$') ]
        
    def sort_by_file(self, a, b):
        return(cmp(a[0], b[0]))

        
    def erScanForFilesSizeDate(self, directory, recursive=0 ):
        if not os.path.exists( directory ):
            return( '%s does not exist.'%(directory), 0 )
        data = []
        for root, dirs, files in os.walk(directory):
            if recursive or root.lower() == directory.lower():
                for file in files:
                    fullfile =  os.path.join( root, file)
                    try:
                        tm = time.localtime( os.path.getmtime(fullfile) )
                        size = '%dk'%(os.path.getsize(fullfile) / 1000 + 1 )
                        atime = '%2d%02d%02d-%02d%02d'%( int(tm[0])%100, int(tm[1]), int(tm[2]), int(tm[3]), int(tm[4]) ) 
                    except:
                        size = atime = '?'
                    data.append( [ file, size, atime ] )
        if len(data):
            data.sort( self.sort_by_file )
        return( 0, data )
        

    def refresh_directory(self):
        self.scan_directory()
        self.update_list(self.gui_list)


    def scan_directory(self):
        self.scan_list = []
        self.gui_list = []
        self.gui_index = -1
        if self.directory:
            error, data = self.erScanForFilesSizeDate( self.directory )
            if error:
                log.error(error)
                return self.gui_list
            for file_item in data:
                filename = file_item[0]
                size     = file_item[1]
                date     = file_item[2]
                date = date.split()[0]
                aPass = False
                for filter in self.pass_filters:
                    if re.search( filter, filename ):
                        aPass = True
                        break
                if not aPass:
                    continue
                self.scan_list.append(filename)
                row = []
                if self.format_active: 
                    act = '  '
                    if self.activeFiles and filename in activeFiles:
                        act = '**'
                        row.append(act)
                row.append(filename)
                if self.format_size: 
                    row.append(size)
                if self.format_date: 
                    row.append(date)
                self.gui_list.append(row)
        return self.gui_list

    def get_selected_files(self):
        self.selected_files = []
        self.get_selected_files_name_only()
        for file in self.selected_files_name_only:
            self.selected_files.append( os.path.join( self.directory, file) )
        return self.selected_files

 
    def get_selected_files_name_only(self):
        self.selected_files_name_only = []
        for indice in self.selected_indices:
            if indice < len(self.scan_list):
                self.selected_files_name_only.append( self.scan_list[indice] )
        return self.selected_files_name_only

class MyTimer(wx.Timer):
    def Notify(self):
        self.ticks += 1
        if not (self.ticks & 0x7):
            log.info('tick')


class PytronixGuiBoard_Type1 (object):
    __metaclass__ = Singleton.Singleton

    def __init__ (self, log_in=0):
        global log
        if 'true_flag' in self.__dict__:
            # Second or more call to Singleton, no need to initialize.
            return
        self.true_flag    = True
        log               = log_in
        Tags.add_tags_as_attributes( self, main_data_tags)        
        Tags.add_tags_as_attributes( self, view_data_tags)        
        self.main_obj     = None
        self.view_objs    = []
        self.view_index   = 0
        self.alsoPrint    = None
        self.gui          = None
        self.bank_mode_on = 0
        

    def mdata(self, tag):
        return( self.main_obj.settings.get(tag, 0))
        
        
    def set_data_objs(self, main_obj, view_objs):
        self.main_obj   = main_obj
        self.view_objs  = view_objs


    def make_gui(self):
        self.main_obj.pre_initialize()
        try:
            self.app = wx.PySimpleApp()
            self.gui = Gui.PytronixGuiWin(None, -1, self.mdata(self.tMainTitle),
                self.mdata(self.tMainPosition), self.mdata(self.tMainSize) )
            for view_obj in self.view_objs:
                view_obj.gui = self.gui
                view_obj.build()
            self.build_buttons()
            self.build_log_output()
            self.gui.buildDone()
        except:
            traceback.print_exc (file=open('error.txt', 'w'))
            print '!!! Please read the error.txt file.'
        for binding in self.main_obj.bindings:
            self.gui.Bind(binding[0], binding[1])
        timer_polling = self.mdata(self.tMainTimerPollingMsec)
        if timer_polling:
            self.timer = MyTimer()
            self.timer.ticks = 0
            self.timer.Start(timer_polling)
        self.main_obj.post_initialize()

    def launch_gui(self):
        #self.make_gui()
        self.app.MainLoop()
        return True


    def build_buttons(self):
        x = self.mdata(self.tMainButtonSizeX)
        y = self.mdata(self.tMainButtonSizeY)
        z = self.mdata(self.tMainViewSizeZ)
        self.bank_mode_on = z > 1
        self.buttons = Gui.PytronixGuiWin_Buttons(self.gui, x, y, z, self.binding_evt_button)
        self.buttons.build()

        for i, view_obj in enumerate(self.view_objs):
            view_obj.build_buttons(self.buttons.array[i])
        if self.bank_mode_on:
            for i, view_obj in enumerate(self.view_objs):
                self.buttons.array[0][ 0][ i].label = view_obj.header
            self.buttons.update_headers()
            self.buttons.selectBank(0)
        else:
            self.buttons.update_bank_0()
        
    def binding_evt_button(self, event):
        x,y = self.buttons.id_mapping.get(event.Id, (0,0))
        #print 'evt_button %d (%d.%d)'%(event.Id, x,y)
        if self.bank_mode_on and x == 0:
            self.view_index = y
            self.buttons.selectBank(self.view_index)
        else:
            log.info('Button not used, View[ %d ] X[ %d ] Y[ %d ]'%(
                self.view_index,x,y))

    def build_log_output(self):
        self.log_output = Gui.PytronixGuiWin_LogOutput(self.gui)
        self.log_output.pointSize = 9
        self.log_output.proportion = 15
        self.log_output.build()


    def print_to_gui (self, text, newline=True):
        if self.gui:
            if newline:
                self.gui.logger('%s\n' % text)
            else:
                self.gui.logger('%s' % text)
        if self.alsoPrint:
            if newline:
                print(text)
            else:
                print(text),
            

       

